@@ -1,3 +1,5 @@
+import collections
+from slugify import slugify
 import itertools
 import logging
 import operator
@@ -47,30 +49,29 @@ class Collection:
     has_archive: Bool
         if `True`, create an archive page with all of the processed pages saved
         as `pages`. default `False`
-    _archive_template: str, optional
+    template: str, optional
         template filename that will be used if `has_archive==True` default: archive.html"
-    _archive_slug: str, optional
+    slug: str, optional
         slug for rendered page if `has_archive == True` default: all_posts
-    _archive_content_type: Type[Page], optional
+    content_type: Type[Page], optional
         content_type for the rendered archive page
     _archive_reverse: Bool, optional
         should the sorted `pages` be listed in reverse order. default: False
 
     """
 
-    engine: typing.Optional[str] = ""
+    engine: typing.Optional[str] = None
     content_items: typing.List[Page] = []
     content_path: str = ""
-    page_content_type: typing.Type[Page] = Page
+    content_type: typing.Type[Page] = Page
     template: str = "page.html"
     includes: typing.List[str] = ["*.md", "*.html"]
     routes: typing.List[str] = [""]
     subcollections: typing.List[str] = []
     has_archive: bool = False
     archive_template: str = "archive.html"
-    archive_slug: str = "all_posts"
-    archive_content_type: Page = Page
     archive_reverse: bool = False
+    archive_sort: typing.Tuple[str] = ('title')
     paginated: bool = False
     items_per_page: int = 10
     title: typing.Optional[str] = ''
@@ -80,13 +81,8 @@ class Collection:
         if not self.title:
             self.title = self.__class__.__name__
 
-    @staticmethod
-    def archive_default_sort(cls):
-        """attribute pulled from a rendered Page to sort `pages`"""
-        return cls.slug
-
     @property
-    def _pages(self) -> typing.List[Page]:
+    def pages(self) -> typing.List[Page]:
         _pages = []
 
         if self.content_items:
@@ -102,102 +98,81 @@ class Collection:
             for pattern in self.includes:
 
                 for filepath in Path(self.content_path).glob(pattern):
-                    page = self.page_content_type.from_content_path(filepath)
+                    page = self.content_type.from_content_path(filepath)
                     page.routes = self.routes
                     page.template = self.template
                     _pages.append(page)
 
         return _pages
 
-
     @property
-    def pages(self) -> typing.List[typing.Union[Page, typing.List]]:
-        """Iterate through set of pages and generate a `Page`-like object for each."""
+    def archive(self):
+        """Create a `Page` object for the pages in the collection"""
 
         sorted_pages = sorted(
-            self._pages,
-            key=lambda p: self.archive_default_sort(p),
+            self.pages,
+            key=lambda p: getattr(p, self.archive_sort),
             reverse=self.archive_reverse,
         )
 
         if self.paginated:
-            return list(more_itertools.chunked(sorted_pages, self.items_per_page))
+            pages = list(more_itertools.chunked(sorted_pages, self.items_per_page))
 
         else:
-            return sorted_pages
+            pages = [sorted_pages]
 
+        class Archive(Page):
+            no_index = True
+            template = self.archive_template
+            routes = [self.routes[0]]
+            title = self.title
+
+        archive_pages = []
+
+        for index, page in enumerate(pages):
+            archive_page =  Archive()
+            archive_page.collection = self
+            archive_page.routes = [self.routes[0]]
+            archive_page.pages = pages[index]
+            archive_page.title = self.title
+            archive_page.page_index = (index, len(pages))
+
+            if self.paginated:
+                archive_page.slug = f'{archive_page.slug}-{index}'
+
+            archive_pages.append(archive_page)
+
+        return archive_pages
 
     @property
-    def archive(self):
-        """Create a `Page` object for the pages in the collection"""
-        archive_page = self.archive_content_type()
-        archive_page.no_index = True
-        archive_page.template = self.archive_template
-        archive_page.slug = self.archive_slug
-        archive_page.engine = ""
-        archive_page.routes = [self.routes[0]]
-        archive_page.pages = self.pages
-        archive_page.title = self.title
+    def get_subcollections(self):
+        subcollections = {}
 
-        if self.paginated:
-            archive_pages = []
+        # get all the values for each of the subcollections
+        for _subcollection in self.subcollections:
+            subcollection_lists = collections.defaultdict(list)
+            subcollections[_subcollection] = []
 
-            for index, page in enumerate(self.pages):
-                archive_page = self.archive_content_type()
-                archive_page.no_index = True
-                archive_page.template = self.archive_template
-                archive_page.slug = f"{self.archive_slug}_{index}"
-                archive_page.engine = ""
-                archive_page.routes = [self.routes[0]]
-                archive_page.pages = self.pages[index]
-                archive_page.title = self.title
-                archive_page.page_index = index
-                archive_pages.append(archive_page)
+            for page in self.pages:
 
+                if attr:=getattr(page, _subcollection, None):
 
-            return archive_pages
+                    if isinstance(attr, list):
 
-        else:
-            archive_page = self.archive_content_type()
-            archive_page.no_index = True
-            archive_page.template = self.archive_template
-            archive_page.slug = f"{self.archive_slug}"
-            archive_page.engine = ""
-            archive_page.routes = [self.routes[0]]
-            archive_page.pages = self.pages
-            archive_page.title = self.title
-            archive_page.page_index = 0
-            return [archive_page]
+                        for attribute_item in attr:
+                            subcollection_lists[attribute_item].append(page)
 
+                    else:
+                        subcollection_lists[attr].append(page)
 
-    @classmethod
-    def from_subcollection(cls, collection, attr, attrval):
-        sub_content_items = []
+            for attr, subcollection in subcollection_lists.items():
 
-        for page in collection._pages:
+                class SubCollection(self.__class__):
+                    content_path = None
+                    content_items = subcollection
+                    title = attr
+                    slug = slugify(attr)
 
-            if attrval in getattr(page, attr, []):
-                sub_content_items.append(page)
+                subcollections[_subcollection].append(SubCollection())
 
-        class SubCollection(Collection):
-            archive_template = collection.archive_template
-            archive_slug = collection.archive_slug
-            archive_content_type = collection.archive_content_type
-            archive_reverse = collection.archive_reverse
-            content_items = sub_content_items
-            has_archive = True
-            routes = [attrval]
-            title = attrval
-
-        return SubCollection()
-
-
-    def subcollection(self, attr):
-        """Returns a list of all of the values in a subcollection"""
-        attrvals = []
-
-        for page in self._pages:
-            if hasattr(page, attr):
-                attrvals.append(getattr(page, attr))
-
-        return attrvals
+        return subcollections
