@@ -1,3 +1,5 @@
+from copy import copy
+from progress.bar import Bar
 import itertools
 import inspect
 import logging
@@ -6,6 +8,7 @@ import shutil
 import typing
 import pendulum
 from pathlib import Path
+from slugify import slugify
 
 
 import more_itertools
@@ -18,25 +21,6 @@ from .links import Link
 from .page import Page
 from .search import Search
 
-def get_subcollections(collection):
-
-    subcollection_set = set() # creates a set to add new subcollections to
-
-    for page in collection._pages:
-
-        for subcollection in collection.subcollections:
-
-            if attr := getattr(page, subcollection, None):
-                if isinstance(attr, list):
-                    for subattr in attr:
-                        subcollection_set.add((subcollection, subattr))
-
-                else:
-                    subcollection_set.add((subcollection, attr))
-
-    return subcollection_set
-
-
 class Site:
     """
     The site stores your pages and collections to be rendered.
@@ -151,32 +135,19 @@ class Site:
                 Collection to parse
         """
         collection = collection_cls()
-        self.collections.update({collection.__class__.__name__: collection})
-        setattr(self, collection.title, collection)
+        self.collections.update({collection.title: collection})
 
-        for page in collection._pages:
+        for page in collection.pages:
+            logging.info(page.title)
             self.route(cls=page)
 
         if collection.has_archive:
+
             for archive in collection.archive:
                 self.route(cls=archive)
 
-        subcollections = get_subcollections(collection)
-
-        for attr, attrval in subcollections:
-            subcollection = Collection.from_subcollection(collection, attr, attrval,)
-            self.collections[attrval] = subcollection
-
-            if attr in self.subcollections.keys():
-                self.subcollections[attr].append(subcollection)
-
-            else:
-                self.subcollections[attr] = [subcollection]
-
-            for archive_page in subcollection.archive:
-                self.route(archive_page)
-
         if collection.feeds:
+
             for feed in collection.feeds:
                 self.register_feed(feed=feed, collection=collection)
 
@@ -185,7 +156,7 @@ class Site:
         extension = self.rss_engine.extension
         _feed = feed()
         _feed.slug = collection.__class__.__name__.lower()
-        _feed.items = [page.rss_feed_item for page in collection._pages]
+        _feed.items = [page.rss_feed_item for page in collection.pages]
         _feed.title = f"{self.SITE_TITLE} - {_feed.title}"
         _feed.link = f"{self.SITE_URL}/{_feed.slug}{extension}"
 
@@ -202,29 +173,57 @@ class Site:
 
 
     def render(self, dry_run: bool = False) -> None:
-        for page in self.routes:
+        for _, collection in self.collections.items():
 
-            if page.engine:
-                engine = page.engine
+            if collection.subcollections:
 
-            else:
-                engine = self.default_engine
+                for subcollection in more_itertools.collapse(
+                        collection.get_subcollections.values()):
 
-            logging.debug(f'{engine=}')
+                        for archive in subcollection.archive:
+                            self.route(archive)
 
-            template_attrs = self.get_public_attributes(page)
 
-            print(template_attrs.get('github', None))
 
-            content = engine.render(page, **template_attrs)
+        route_count = len(self.routes)
 
-            for route in page.routes:
-                logging.debug(f"starting on {route=}")
-                route = self.output_path.joinpath(route.strip("/"))
+        with Bar(
+                f'Rendering {route_count} Pages',
+                max=route_count,
+                suffix='%(percent).1f%% - %(elapsed_td)s') as bar:
+            for page in self.routes:
+                suffix='%(percent).1f%% - %(elapsed_td)s'
+                bar.suffix = suffix + f' ({page.title})'
+
+                if page.engine:
+                    engine = page.engine
+
+                else:
+                    engine = self.default_engine
+
+                logging.debug(f'{engine=}')
+
+                template_attrs = self.get_public_attributes(page)
+
+                content = engine.render(page, **template_attrs)
+
+                route = self.output_path.joinpath(page.routes[0].strip("/"))
                 route.mkdir(exist_ok=True)
                 filename = Path(page.slug).with_suffix(engine.extension)
                 filepath = route.joinpath(filename)
                 filepath.write_text(content)
+                logging.info(f'{filepath=} written!')
+
+                if len(page.routes) > 1:
+
+                    for new_route in page.routes[1:]:
+                        new_route = self.output_path.joinpath(new_route.strip("/"))
+                        new_route.mkdir(exist_ok=True)
+                        new_filepath = new_route.joinpath(filename)
+                        shutil.copy(filepath, new_filepath)
+                        logging.info(f'{new_filepath=} written!')
+
+                bar.next()
 
         if self.search:
             search_pages = filter(lambda x: x.no_index == False, self.routes)
